@@ -5,20 +5,24 @@ from __future__ import annotations
 import os
 import re
 import string
+from dataclasses import dataclass
 from typing import TYPE_CHECKING
 
+from mkdocs.exceptions import PluginError
 from wcmatch import glob
 
 from mkdocs_include_markdown_plugin import process
 
 
+@dataclass
+class DirectiveBoolArgument:  # noqa: D101
+    value: bool
+    regex: re.Pattern[str]
+
+
 if TYPE_CHECKING:
     from typing import TypedDict
 
-    class DirectiveBoolArgument(TypedDict):  # noqa: D101
-        value: bool
-        regex: re.Pattern[str]
-
     DirectiveBoolArgumentsDict = dict[str, DirectiveBoolArgument]
 
     DefaultValues = TypedDict(
@@ -30,8 +34,8 @@ if TYPE_CHECKING:
             'comments': bool,
             'rewrite-relative-urls': bool,
             'heading-offset': int,
-            'start': str,
-            'end': str,
+            'start': str | None,
+            'end': str | None,
         },
     )
 
@@ -102,9 +106,9 @@ def parse_filename_argument(
         match: re.Match[str],
 ) -> tuple[str | None, str | None]:
     """Return filename argument matched by ``match``."""
-    raw_filename = match.group('double_quoted_filename')
+    raw_filename = match['double_quoted_filename']
     if raw_filename is None:
-        raw_filename = match.group('single_quoted_filename')
+        raw_filename = match['single_quoted_filename']
         if raw_filename is None:
             filename = None
         else:
@@ -116,9 +120,9 @@ def parse_filename_argument(
 
 def parse_string_argument(match: re.Match[str]) -> str | None:
     """Return the string argument matched by ``match``."""
-    value = match.group(1)
+    value = match[1]
     if value is None:
-        value = match.group(3)
+        value = match[3]
         if value is not None:
             value = value.replace("\\'", "'")
     else:
@@ -152,20 +156,18 @@ def parse_bool_options(
 
     bool_options: dict[str, DirectiveBoolArgument] = {}
     for option_name in option_names:
-        bool_options[option_name] = {
-            'value': defaults[option_name],  # type: ignore
-            'regex': ARGUMENT_REGEXES[option_name],
-        }
+        bool_options[option_name] = DirectiveBoolArgument(
+            value=defaults[option_name],  # type: ignore
+            regex=ARGUMENT_REGEXES[option_name],
+        )
 
     for arg_name, arg in bool_options.items():
-        bool_arg_match = arg['regex'].search(arguments_string)
+        bool_arg_match = arg.regex.search(arguments_string)
         if bool_arg_match is None:
             continue
         try:
-            bool_options[arg_name]['value'] = TRUE_FALSE_STR_BOOL[
-                bool_arg_match.group(
-                    1,
-                ) or TRUE_FALSE_BOOL_STR[arg['value']]
+            bool_options[arg_name].value = TRUE_FALSE_STR_BOOL[
+                bool_arg_match[1] or TRUE_FALSE_BOOL_STR[arg.value]
             ]
         except KeyError:
             invalid_args.append(arg_name)
@@ -173,19 +175,19 @@ def parse_bool_options(
 
 
 def resolve_file_paths_to_include(
-    filename_or_url: str,
-    includer_page_src_path: str,
+    include_string: str,
+    includer_page_src_path: str | None,
     docs_dir: str,
     ignore_paths: list[str],
 ) -> tuple[list[str], bool]:
     """Resolve the file paths to include for a directive."""
-    if process.is_url(filename_or_url):
-        return [filename_or_url], True
+    if process.is_url(include_string):
+        return [include_string], True
 
-    if process.is_absolute_path(filename_or_url):
-        if os.name == 'nt':
+    if process.is_absolute_path(include_string):
+        if os.name == 'nt':  # pragma: nt cover
             # Windows
-            fpath = os.path.normpath(filename_or_url)
+            fpath = os.path.normpath(include_string)
             if not os.path.isfile(fpath):
                 return [], False
 
@@ -194,13 +196,19 @@ def resolve_file_paths_to_include(
             ), False
         return process.filter_paths(
             glob.iglob(
-                os.path.normpath(filename_or_url),
+                os.path.normpath(include_string),
                 flags=GLOB_FLAGS,
             ),
             ignore_paths,
         ), False
 
-    if process.is_relative_path(filename_or_url):
+    if process.is_relative_path(include_string):
+        if includer_page_src_path is None:  # pragma: no cover
+            raise PluginError(
+                'Relative paths are not allowed when the includer page'
+                ' source path is not provided. The include string'
+                f" '{include_string}' is located inside a generated page.",
+            )
         root_dir = os.path.abspath(
             os.path.dirname(includer_page_src_path),
         )
@@ -208,7 +216,7 @@ def resolve_file_paths_to_include(
             (
                 os.path.normpath(os.path.join(root_dir, fp))
                 for fp in glob.iglob(
-                    filename_or_url,
+                    include_string,
                     flags=GLOB_FLAGS,
                     root_dir=root_dir,
                 )
@@ -220,7 +228,7 @@ def resolve_file_paths_to_include(
         (
             os.path.normpath(os.path.join(docs_dir, fp))
             for fp in glob.iglob(
-                filename_or_url,
+                include_string,
                 flags=GLOB_FLAGS,
                 root_dir=docs_dir,
             )
@@ -231,7 +239,7 @@ def resolve_file_paths_to_include(
 
 def resolve_file_paths_to_exclude(
     exclude_string: str,
-    includer_page_src_path: str,
+    includer_page_src_path: str | None,
     docs_dir: str,
 ) -> list[str]:
     """Resolve the file paths to exclude for a directive."""
@@ -240,6 +248,12 @@ def resolve_file_paths_to_exclude(
         return glob.glob(exclude_string, flags=GLOB_FLAGS)
 
     if process.is_relative_path(exclude_string):
+        if includer_page_src_path is None:  # pragma: no cover
+            raise PluginError(
+                'Relative paths are not allowed when the includer page'
+                ' source path is not provided. The exclude string'
+                f" '{exclude_string}' is located inside a generated page.",
+            )
         root_dir = os.path.abspath(
             os.path.dirname(includer_page_src_path),
         )
