@@ -3,6 +3,7 @@
 from __future__ import annotations
 
 from collections.abc import Callable
+from functools import cached_property
 from typing import TYPE_CHECKING, Any
 
 from mkdocs.exceptions import PluginError
@@ -11,12 +12,14 @@ from mkdocs.plugins import BasePlugin, event_priority
 
 
 if TYPE_CHECKING:
+    import re
+
     from mkdocs.config.defaults import MkDocsConfig
     from mkdocs.structure.files import Files
     from mkdocs.structure.pages import Page
 
-from mkdocs_include_markdown_plugin.cache import initialize_cache
-from mkdocs_include_markdown_plugin.config import CONFIG_SCHEME
+from mkdocs_include_markdown_plugin.cache import Cache, initialize_cache
+from mkdocs_include_markdown_plugin.config import PluginConfig
 from mkdocs_include_markdown_plugin.directive import create_include_tag
 from mkdocs_include_markdown_plugin.event import (
     on_page_markdown as _on_page_markdown,
@@ -24,23 +27,13 @@ from mkdocs_include_markdown_plugin.event import (
 from mkdocs_include_markdown_plugin.files_watcher import FilesWatcher
 
 
-class IncludeMarkdownPlugin(BasePlugin):
-    config_scheme = CONFIG_SCHEME
+class IncludeMarkdownPlugin(BasePlugin[PluginConfig]):
+    _cache: Cache | None = None
+    _server: LiveReloadServer | None = None
 
     def on_config(self, config: MkDocsConfig) -> MkDocsConfig:
-        self.config['_include_tag'] = create_include_tag(
-            self.config['opening_tag'],
-            self.config['closing_tag'],
-        )
-
-        self.config['_include_markdown_tag'] = create_include_tag(
-            self.config['opening_tag'],
-            self.config['closing_tag'],
-            tag='include-markdown',
-        )
-
-        if self.config['cache'] > 0:
-            cache = initialize_cache(self.config['cache'])
+        if self.config.cache > 0:
+            cache = initialize_cache(self.config.cache)
             if cache is None:
                 raise PluginError(
                     'The "platformdirs" package is required to use the'
@@ -49,26 +42,46 @@ class IncludeMarkdownPlugin(BasePlugin):
                     ' extra to install it.',
                 )
             cache.clean()
-            self.config['_cache'] = cache
-
-        self.config['_files_watcher'] = None
-        self.config['_server'] = None
+            self._cache = cache
 
         return config
 
-    def _watch_included_files(self) -> None:  # pragma: no cover
-        # unwatch previous watched files not needed anymore
-        for filepath in self.config['_files_watcher'].prev_included_files:
-            if filepath not in self.config['_files_watcher'].included_files:
-                self.config['_server'].unwatch(filepath)
-        self.config['_files_watcher'].prev_included_files = (
-            self.config['_files_watcher'].included_files[:]
+    @cached_property
+    def _include_tag(self) -> re.Pattern[str]:
+        return create_include_tag(
+            self.config.opening_tag,
+            self.config.closing_tag,
         )
 
+    @cached_property
+    def _include_markdown_tag(self) -> re.Pattern[str]:
+        return create_include_tag(
+            self.config.opening_tag,
+            self.config.closing_tag,
+            tag='include-markdown',
+        )
+
+    @cached_property
+    def _files_watcher(self) -> FilesWatcher:
+        return FilesWatcher()
+
+    def _update_watched_files(self) -> None:  # pragma: no cover
+        """Function executed on server reload.
+
+        At this execution point, the ``self._server`` attribute must be set.
+        """
+        watcher, server = self._files_watcher, self._server
+
+        # unwatch previous watched files not needed anymore
+        for file_path in watcher.prev_included_files:
+            if file_path not in watcher.included_files:
+                server.unwatch(file_path)  # type: ignore
+        watcher.prev_included_files = watcher.included_files[:]
+
         # watch new included files
-        for filepath in self.config['_files_watcher'].included_files:
-            self.config['_server'].watch(filepath, recursive=False)
-        self.config['_files_watcher'].included_files = []
+        for file_path in watcher.included_files:
+            server.watch(file_path, recursive=False)  # type: ignore
+        watcher.included_files = []
 
     def on_page_content(
             self,
@@ -77,8 +90,8 @@ class IncludeMarkdownPlugin(BasePlugin):
             config: MkDocsConfig,  # noqa: ARG002
             files: Files,  # noqa: ARG002
     ) -> str:
-        if self.config['_server'] is not None:  # pragma: no cover
-            self._watch_included_files()
+        if self._server is not None:  # pragma: no cover
+            self._update_watched_files()
         return html
 
     def on_serve(
@@ -87,9 +100,9 @@ class IncludeMarkdownPlugin(BasePlugin):
             config: MkDocsConfig,  # noqa: ARG002
             builder: Callable[[Any], Any],  # noqa: ARG002
     ) -> None:
-        if self.config['_server'] is None:  # pragma: no cover
-            self.config['_server'] = server
-            self._watch_included_files()
+        if self._server is None:  # pragma: no cover
+            self._server = server
+            self._update_watched_files()
 
     @event_priority(100)
     def on_page_markdown(
@@ -99,11 +112,9 @@ class IncludeMarkdownPlugin(BasePlugin):
             config: MkDocsConfig,
             files: Files,  # noqa: ARG002
     ) -> str:
-        if self.config['_files_watcher'] is None:
-            self.config['_files_watcher'] = FilesWatcher()
         return _on_page_markdown(
             markdown,
             page,
-            config['docs_dir'],
-            config=self.config,
+            config.docs_dir,
+            plugin=self,
         )
